@@ -52,9 +52,9 @@ but tends to yield better packings.
 
 This procedure has originally been developed and implemented by Mr. Rui ZHAO
 (赵睿), <zr1329142665@163.com> a Master's student at the Institute of Applied
-Optimization (应用优化研究所, http://iao.hfuu.edu.cn) of the School of
+Optimization (应用优化研究所) of the School of
 Artificial Intelligence and Big Data (人工智能与大数据学院) at Hefei University
-(合肥学院) in Hefei, Anhui, China (中国安徽省合肥市) under the supervision of
+(合肥大学) in Hefei, Anhui, China (中国安徽省合肥市) under the supervision of
 Prof. Dr. Thomas Weise (汤卫思教授).
 
 1. Dequan Liu and Hongfei Teng. An Improved BL-Algorithm for Genetic Algorithm
@@ -369,7 +369,7 @@ def _decode(x: np.ndarray, y: np.ndarray, instance: np.ndarray,
 
 # If the object is not fully inside the current bin, we move to a new bin.
         if (y[i, IDX_RIGHT_X] > bin_width) or (y[i, IDX_TOP_Y] > bin_height):
-            bin_id = bin_id + 1  # step to the next bin
+            bin_id += 1  # step to the next bin
             bin_start = i  # set the starting index of the bin
             y[i, IDX_LEFT_X] = 0  # the object goes to the left end of the bin
             y[i, IDX_BOTTOM_Y] = 0  # the object goes to the bottom of the bin
