@@ -77,9 +77,9 @@ already placed object per bin.
 
 This procedure has originally been developed and implemented by Mr. Rui ZHAO
 (赵睿), <zr1329142665@163.com>, a Master's student at the Institute of Applied
-Optimization (应用优化研究所, http://iao.hfuu.edu.cn) of the School of
+Optimization (应用优化研究所) of the School of
 Artificial Intelligence and Big Data (人工智能与大数据学院) at Hefei University
-(合肥学院) in Hefei, Anhui, China (中国安徽省合肥市) under the supervision of
+(合肥大学) in Hefei, Anhui, China (中国安徽省合肥市) under the supervision of
 Prof. Dr. Thomas Weise (汤卫思教授).
 
 1. Dequan Liu and Hongfei Teng. An Improved BL-Algorithm for Genetic Algorithm
@@ -432,7 +432,7 @@ def _decode(x: np.ndarray, y: np.ndarray, instance: np.ndarray,
         if not_found:  # we did not find a spot in any of the bins
             bin_starts[bin_id] = i  # set the starting index of the new bin
             bin_ends[bin_id] = i + 1  # set the end index of the new bin
-            bin_id = bin_id + 1  # step to the next bin
+            bin_id += 1  # step to the next bin
             y[i, IDX_LEFT_X] = 0  # the object goes to the left end of the bin
             y[i, IDX_BOTTOM_Y] = 0  # the object goes to the bottom of the bin
             y[i, IDX_RIGHT_X] = w  # so its right end is its width
