@@ -3,11 +3,11 @@
 import argparse
 from typing import Callable, Final, cast
 
-from moptipy.algorithms.so.fea1plus1 import FEA1plus1
+from moptipy.algorithms.so.ffa.fea1plus1 import FEA1plus1
 from moptipy.algorithms.so.rls import RLS
 from moptipy.api.encoding import Encoding
 from moptipy.api.execution import Execution
-from moptipy.api.experiment import Parallelism, run_experiment
+from moptipy.api.experiment import run_experiment
 from moptipy.api.objective import Objective
 from moptipy.operators.signed_permutations.op0_shuffle_and_flip import (
     Op0ShuffleAndFlip,
@@ -101,7 +101,7 @@ def run(base_dir: str, n_runs: int = 23) -> None:
         inst for inst in Instance.list_resources()
         if inst.startswith(("b", "a"))]
     inst_creators: list[Callable[[], Instance]] = [cast(
-        Callable[[], Instance], lambda __s=_s: Instance.from_resource(__s))
+        "Callable[[], Instance]", lambda __s=_s: Instance.from_resource(__s))
         for _s in instances]
     namer: Final[Instance] = Instance.from_resource(instances[0])
 
@@ -116,12 +116,13 @@ def run(base_dir: str, n_runs: int = 23) -> None:
                 base_dir=encoding_dir,
                 instances=inst_creators,
                 setups=[
-                    cast(Callable, lambda ins, _e=encoding, _o=objective: rls(
-                        ins, _e, _o)),
-                    cast(Callable, lambda ins, _e=encoding, _o=objective: fea(
-                        ins, _e, _o))],
+                    cast("Callable",
+                         lambda ins, _e=encoding, _o=objective: rls(
+                             ins, _e, _o)),
+                    cast("Callable",
+                         lambda ins, _e=encoding, _o=objective: fea(
+                             ins, _e, _o))],
                 n_runs=n_runs,
-                n_threads=Parallelism.ACCURATE_TIME_MEASUREMENTS,
                 perform_warmup=True,
                 perform_pre_warmup=True)
 
