@@ -11,7 +11,7 @@ instances.
 import argparse
 from dataclasses import dataclass
 from math import isfinite
-from typing import Any, Callable, Final, Iterable, Mapping, cast
+from typing import Any, Callable, Final, Generator, Iterable, Mapping, cast
 
 from moptipy.api.objective import Objective
 from moptipy.evaluation.base import (
@@ -23,21 +23,22 @@ from moptipy.evaluation.end_results import EndResult
 from moptipy.evaluation.end_results import from_logs as er_from_logs
 from moptipy.evaluation.log_parser import LogParser
 from pycommons.ds.immutable_map import immutable_mapping
+from pycommons.ds.sequences import reiterable
 from pycommons.io.console import logger
 from pycommons.io.csv import (
     SCOPE_SEPARATOR,
     csv_column,
-    csv_read,
     csv_scope,
     csv_select_scope,
-    csv_write,
 )
+from pycommons.io.csv import CsvReader as CsvReaderBase
+from pycommons.io.csv import CsvWriter as CsvWriterBase
 from pycommons.io.path import Path, file_path, line_writer
 from pycommons.strings.string_conv import (
     num_to_str,
     str_to_num,
 )
-from pycommons.types import check_int_range, reiterable, type_error
+from pycommons.types import check_int_range, type_error
 
 from moptipyapps.binpacking2d.instance import (
     Instance,
@@ -212,7 +213,7 @@ class PackingResult(EvaluationDataElement):
                     f"it is required that {lll}<=f<={uuu}, but got "
                     f"{lower}, {value}, and {upper}.")
 
-        bins: Final[int | None] = cast(int, objectives[BIN_COUNT_NAME]) \
+        bins: Final[int | None] = cast("int", objectives[BIN_COUNT_NAME]) \
             if BIN_COUNT_NAME in objectives else None
         for name, value in bin_bounds.items():
             if not isinstance(name, str):
@@ -354,37 +355,29 @@ def from_single_log(  # pylint: disable=W0102
     :return: the packing result
     """
     the_file_path = file_path(file)
-    end_results: Final[list[EndResult]] = []
-    er_from_logs(the_file_path, end_results.append)
-    if len(end_results) != 1:
-        raise ValueError(
-            f"needs one end result record in file {the_file_path!r}, "
-            f"but got {end_results}.")
-
+    end_result: Final[EndResult] = next(er_from_logs(the_file_path))
     packing = Packing.from_log(the_file_path)
     if not isinstance(packing, Packing):
         raise type_error(packing, f"packing from {file!r}", Packing)
     return from_packing_and_end_result(
-        end_result=end_results[0], packing=packing,
+        end_result=end_result, packing=packing,
         objectives=objectives, bin_bounds=bin_bounds, cache=cache)
 
 
 def from_logs(  # pylint: disable=W0102
         directory: str,
-        consumer: Callable[[PackingResult], None],
         objectives: Iterable[Callable[[Instance], Objective]] =
         DEFAULT_OBJECTIVES,
         bin_bounds: Mapping[str, Callable[[Instance], int]]
-        = _DEFAULT_BOUNDS) -> None:
+        = _DEFAULT_BOUNDS) -> Generator[PackingResult, None, None]:
     """
     Parse a directory recursively to get all packing results.
 
     :param directory: the directory to parse
-    :param consumer: the consumer for receiving the results
     :param objectives: the objective function factories
     :param bin_bounds: the bin bounds calculators
     """
-    __LogParser(consumer, objectives, bin_bounds).parse_dir(directory)
+    return __LogParser(objectives, bin_bounds).parse(directory)
 
 
 def to_csv(results: Iterable[PackingResult], file: str) -> Path:
@@ -399,182 +392,147 @@ def to_csv(results: Iterable[PackingResult], file: str) -> Path:
     logger(f"Writing packing results to CSV file {path!r}.")
     path.ensure_parent_dir_exists()
     with path.open_for_write() as wt:
-        csv_write(
-            data=sorted(results), consumer=line_writer(wt),
-            setup=CsvWriter().setup,
-            get_column_titles=CsvWriter.get_column_titles,
-            get_row=CsvWriter.get_row,
-            get_header_comments=CsvWriter.get_header_comments,
-            get_footer_comments=CsvWriter.get_footer_comments,
-            get_footer_bottom_comments=CsvWriter.get_footer_bottom_comments)
+        consumer: Final[Callable[[str], None]] = line_writer(wt)
+        for p in CsvWriter.write(sorted(results)):
+            consumer(p)
     logger(f"Done writing packing results to CSV file {path!r}.")
     return path
 
 
-def from_csv(file: str,
-             consumer: Callable[[PackingResult], None]) -> None:
+def from_csv(file: str) -> Iterable[PackingResult]:
     """
     Load the packing results from a CSV file.
 
     :param file: the file to read from
-    :param consumer: the consumer for the results
+    :returns: the iterable with the packing result
     """
-    if not callable(consumer):
-        raise type_error(consumer, "consumer", call=True)
     path: Final[Path] = file_path(file)
     logger(f"Now reading CSV file {path!r}.")
     with path.open_for_read() as rd:
-        csv_read(rows=rd,
-                 setup=CsvReader,
-                 parse_row=CsvReader.parse_row,
-                 consumer=consumer)
+        yield from CsvReader.read(rd)
     logger(f"Done reading CSV file {path!r}.")
 
 
-class CsvWriter:
+class CsvWriter(CsvWriterBase[PackingResult]):
     """A class for CSV writing of :class:`PackingResult`."""
 
-    def __init__(self, scope: str | None = None) -> None:
+    def __init__(self, data: Iterable[PackingResult],
+                 scope: str | None = None) -> None:
         """
         Initialize the csv writer.
 
+        :param data: the data to write
         :param scope: the prefix to be pre-pended to all columns
         """
-        #: an optional scope
-        self.scope: Final[str | None] = (
-            str.strip(scope)) if scope is not None else None
-
-        #: has this writer been set up?
-        self.__setup: bool = False
-        #: the end result writer
-        self.__er: Final[ErCsvWriter] = ErCsvWriter(scope)
-        #: the bin bounds
-        self.__bin_bounds: list[str] | None = None
-        #: the objectives
-        self.__objectives: list[str] | None = None
-
-    def setup(self, data: Iterable[PackingResult]) -> "CsvWriter":
-        """
-        Set up this csv writer based on existing data.
-
-        :param data: the data to setup with
-        :returns: this writer
-        """
-        if self.__setup:
-            raise ValueError("CSV writer has already been set up.")
-        self.__setup = True
-
         data = reiterable(data)
-        self.__er.setup(pr.end_result for pr in data)
+        super().__init__(data, scope)
+        #: the end result writer
+        self.__er: Final[ErCsvWriter] = ErCsvWriter((
+            pr.end_result for pr in data), scope)
 
         bin_bounds_set: Final[set[str]] = set()
         objectives_set: Final[set[str]] = set()
         for pr in data:
             bin_bounds_set.update(pr.bin_bounds.keys())
             objectives_set.update(pr.objectives.keys())
-        if set.__len__(bin_bounds_set) > 0:
-            self.__bin_bounds = sorted(bin_bounds_set)
-        if set.__len__(objectives_set) > 0:
-            self.__objectives = sorted(objectives_set)
+        #: the bin bounds
+        self.__bin_bounds: Final[list[str] | None] = None \
+            if set.__len__(bin_bounds_set) <= 0 else sorted(bin_bounds_set)
+        #: the objectives
+        self.__objectives: Final[list[str] | None] = None \
+            if set.__len__(objectives_set) <= 0 else sorted(objectives_set)
 
-        return self
-
-    def get_column_titles(self, dest: Callable[[str], None]) -> None:
+    def get_column_titles(self) -> Iterable[str]:
         """
         Get the column titles.
 
-        :param dest: the destination string consumer
+        :returns: the column titles
         """
         p: Final[str] = self.scope
-        self.__er.get_column_titles(dest)
+        yield from self.__er.get_column_titles()
 
-        dest(csv_scope(p, KEY_BIN_HEIGHT))
-        dest(csv_scope(p, KEY_BIN_WIDTH))
-        dest(csv_scope(p, KEY_N_ITEMS))
-        dest(csv_scope(p, KEY_N_DIFFERENT_ITEMS))
+        yield csv_scope(p, KEY_BIN_HEIGHT)
+        yield csv_scope(p, KEY_BIN_WIDTH)
+        yield csv_scope(p, KEY_N_ITEMS)
+        yield csv_scope(p, KEY_N_DIFFERENT_ITEMS)
         if self.__bin_bounds:
             for b in self.__bin_bounds:
-                dest(csv_scope(p, b))
+                yield csv_scope(p, b)
         if self.__objectives:
             for o in self.__objectives:
                 oo: str = csv_scope(p, o)
-                dest(csv_scope(oo, _OBJECTIVE_LOWER))
-                dest(oo)
-                dest(csv_scope(oo, _OBJECTIVE_UPPER))
+                yield csv_scope(oo, _OBJECTIVE_LOWER)
+                yield oo
+                yield csv_scope(oo, _OBJECTIVE_UPPER)
 
-    def get_row(self, data: PackingResult,
-                dest: Callable[[str], None]) -> None:
+    def get_row(self, data: PackingResult) -> Iterable[str]:
         """
         Render a single packing result record to a CSV row.
 
         :param data: the end result record
-        :param dest: the string consumer
+        :returns: the iterable with the row data
         """
-        self.__er.get_row(data.end_result, dest)
-        dest(repr(data.bin_height))
-        dest(repr(data.bin_width))
-        dest(repr(data.n_items))
-        dest(repr(data.n_different_items))
+        yield from self.__er.get_row(data.end_result)
+        yield repr(data.bin_height)
+        yield repr(data.bin_width)
+        yield repr(data.n_items)
+        yield repr(data.n_different_items)
         if self.__bin_bounds:
             for bb in self.__bin_bounds:
-                dest(repr(data.bin_bounds[bb])
-                     if bb in data.bin_bounds else "")
+                yield (repr(data.bin_bounds[bb])
+                       if bb in data.bin_bounds else "")
         if self.__objectives:
             for ob in self.__objectives:
                 ox = csv_scope(ob, _OBJECTIVE_LOWER)
-                dest(num_to_str(data.objective_bounds[ox])
-                     if ox in data.objective_bounds else "")
-                dest(num_to_str(data.objectives[ob])
-                     if ob in data.objectives else "")
+                yield (num_to_str(data.objective_bounds[ox])
+                       if ox in data.objective_bounds else "")
+                yield (num_to_str(data.objectives[ob])
+                       if ob in data.objectives else "")
                 ox = csv_scope(ob, _OBJECTIVE_UPPER)
-                dest(num_to_str(data.objective_bounds[ox])
-                     if ox in data.objective_bounds else "")
+                yield (num_to_str(data.objective_bounds[ox])
+                       if ox in data.objective_bounds else "")
 
-    def get_header_comments(self, dest: Callable[[str], None]) -> None:
+    def get_header_comments(self) -> Iterable[str]:
         """
         Get any possible header comments.
 
-        :param dest: the destination
+        :returns: the header comments
         """
-        dest("End Results of Bin Packing Experiments")
-        dest("See the description at the bottom of the file.")
+        return ("End Results of Bin Packing Experiments",
+                "See the description at the bottom of the file.")
 
-    def get_footer_comments(self, dest: Callable[[str], None]) -> None:
+    def get_footer_comments(self) -> Iterable[str]:
         """
         Get any possible footer comments.
 
-        :param dest: the destination
+        :return: the footer comments
         """
-        self.__er.get_footer_comments(dest)
-        dest("")
+        yield from self.__er.get_footer_comments()
+        yield ""
         p: Final[str | None] = self.scope
         if self.__bin_bounds:
             for bb in self.__bin_bounds:
-                dest(f"{csv_scope(p, bb)} is a lower bound "
-                     "for the number of bins.")
+                yield (f"{csv_scope(p, bb)} is a lower bound "
+                       f"for the number of bins.")
         if self.__objectives:
             for obb in self.__objectives:
                 ob: str = csv_scope(p, obb)
                 ox: str = csv_scope(ob, _OBJECTIVE_LOWER)
-                dest(f"{ox}: a lower bound of the {ob} objective function.")
-                dest(f"{ob}: one of the possible objective functions for the "
-                     "two-dimensional bin packing problem.")
+                yield f"{ox}: a lower bound of the {ob} objective function."
+                yield (f"{ob}: one of the possible objective functions for "
+                       "the two-dimensional bin packing problem.")
                 ox = csv_scope(ob, _OBJECTIVE_UPPER)
-                dest(f"{ox}: an upper bound of the {ob} objective function.")
+                yield f"{ox}: an upper bound of the {ob} objective function."
 
-    def get_footer_bottom_comments(self, dest: Callable[[str], None]) -> None:
-        """
-        Get the bottom footer comments.
-
-        :param dest: the destination
-        """
-        motipyapps_footer_bottom_comments(
-            self, dest, "The packing data is assembled using module "
-                        "moptipyapps.binpacking2d.packing_statistics.")
-        ErCsvWriter.get_footer_bottom_comments(self.__er, dest)
+    def get_footer_bottom_comments(self) -> Iterable[str]:
+        """Get the bottom footer comments."""
+        yield from motipyapps_footer_bottom_comments(
+            self, "The packing data is assembled using module "
+                  "moptipyapps.binpacking2d.packing_statistics.")
+        yield from ErCsvWriter.get_footer_bottom_comments(self.__er)
 
 
-class CsvReader:
+class CsvReader(CsvReaderBase):
     """A class for CSV reading of :class:`PackingResult` instances."""
 
     def __init__(self, columns: dict[str, int]) -> None:
@@ -583,7 +541,7 @@ class CsvReader:
 
         :param columns: the columns
         """
-        super().__init__()
+        super().__init__(columns)
         #: the end result csv reader
         self.__er: Final[ErCsvReader] = ErCsvReader(columns)
         #: the index of the n-items column
@@ -649,28 +607,22 @@ class CsvReader:
              if str.__len__(data[i]) > 0})
 
 
-class __LogParser(LogParser):
+class __LogParser(LogParser[PackingResult]):
     """The internal log parser class."""
 
-    def __init__(self, consumer: Callable[[PackingResult], None],
-                 objectives: Iterable[Callable[[Instance], Objective]],
+    def __init__(self, objectives: Iterable[Callable[[Instance], Objective]],
                  bin_bounds: Mapping[str, Callable[[Instance], int]]) -> None:
         """
         Parse a directory recursively to get all packing results.
 
-        :param consumer: the consumer for receiving the results
         :param objectives: the objective function factories
         :param bin_bounds: the bin bounds calculators
         """
         super().__init__()
-        if not callable(consumer):
-            raise type_error(consumer, "consumer", call=True)
         if not isinstance(objectives, Iterable):
             raise type_error(objectives, "objectives", Iterable)
         if not isinstance(bin_bounds, Mapping):
             raise type_error(bin_bounds, "bin_bounds", Mapping)
-        #: the internal consumer
-        self.__consumer: Final[Callable[[PackingResult], None]] = consumer
         #: the objectives holder
         self.__objectives: Final[
             Iterable[Callable[[Instance], Objective]]] = objectives
@@ -682,20 +634,17 @@ class __LogParser(LogParser):
             str, tuple[Mapping[str, int], tuple[
                 Objective, ...], Mapping[str, int | float]]]] = {}
 
-    def parse_file(self, path: str) -> bool:
+    def _parse_file(self, file: Path) -> PackingResult:
         """
         Parse a log file.
 
-        :param path: the path to the log file
-        :return: `True`
+        :param file: the file path
+        :return: the parsed result
         """
-        self.__consumer(from_single_log(path, self.__objectives,
-                                        self.__bin_bounds, self.__cache))
-        return True
+        return from_single_log(file, self.__objectives, self.__bin_bounds,
+                               self.__cache)
 
 
-# Evaluate an experiment from the command line
-
 # Run log files to end results if executed as script
 if __name__ == "__main__":
     parser: Final[argparse.ArgumentParser] = moptipyapps_argparser(
@@ -711,6 +660,4 @@ if __name__ == "__main__":
         type=Path, nargs="?", default="./evaluation/end_results.txt")
     args: Final[argparse.Namespace] = parser.parse_args()
 
-    packing_results: Final[list[PackingResult]] = []
-    from_logs(args.source, packing_results.append)
-    to_csv(packing_results, args.dest)
+    to_csv(from_logs(args.source), args.dest)
