@@ -3,7 +3,7 @@ import argparse
 import os.path
 from dataclasses import dataclass
 from math import isfinite
-from typing import Any, Callable, Final, Iterable, Mapping, cast
+from typing import Any, Callable, Final, Generator, Iterable, Mapping, cast
 
 from moptipy.evaluation.base import (
     KEY_N,
@@ -22,22 +22,23 @@ from moptipy.utils.strings import (
     num_to_str,
 )
 from pycommons.ds.immutable_map import immutable_mapping
+from pycommons.ds.sequences import reiterable
 from pycommons.io.console import logger
 from pycommons.io.csv import (
     SCOPE_SEPARATOR,
     csv_column,
-    csv_read,
     csv_scope,
     csv_select_scope,
-    csv_write,
 )
+from pycommons.io.csv import CsvReader as CsvReaderBase
+from pycommons.io.csv import CsvWriter as CsvWriterBase
 from pycommons.io.path import Path, file_path, line_writer
 from pycommons.math.sample_statistics import CsvReader as SsCsvReader
 from pycommons.math.sample_statistics import CsvWriter as SsCsvWriter
 from pycommons.math.sample_statistics import SampleStatistics
 from pycommons.math.sample_statistics import from_samples as ss_from_samples
 from pycommons.strings.string_conv import str_to_num
-from pycommons.types import check_int_range, reiterable, type_error
+from pycommons.types import check_int_range, type_error
 
 from moptipyapps.binpacking2d.objectives.bin_count import BIN_COUNT_NAME
 from moptipyapps.binpacking2d.packing_result import (
@@ -154,7 +155,7 @@ class PackingStatistics(EvaluationDataElement):
                         f"it is required that {lll}<=f<={uuu}, but got "
                         f"{lower}, {value}, and {upper}.")
         bins: Final[SampleStatistics | None] = cast(
-            SampleStatistics, objectives[BIN_COUNT_NAME]) \
+            "SampleStatistics", objectives[BIN_COUNT_NAME]) \
             if BIN_COUNT_NAME in objectives else None
         for name2, value2 in bin_bounds.items():
             if not isinstance(name2, str):
@@ -191,20 +192,16 @@ class PackingStatistics(EvaluationDataElement):
         return self.end_statistics._tuple()
 
 
-def from_packing_results(
-        results: Iterable[PackingResult],
-        consumer: Callable[[PackingStatistics], None]) -> None:
+def from_packing_results(results: Iterable[PackingResult]) \
+        -> Generator[PackingStatistics, None, None]:
     """
     Create packing statistics from a sequence of packing results.
 
     :param results: the packing results
-    :param consumer: the consumer receiving the created packing
-        statistics
+    :returns: a sequence of packing statistics
     """
     if not isinstance(results, Iterable):
         raise type_error(results, "results", Iterable)
-    if not callable(consumer):
-        raise type_error(consumer, "consumer", call=True)
     groups: Final[dict[tuple[str, str, str, str], list[PackingResult]]] \
         = {}
     objectives_set: set[str] = set()
@@ -279,11 +276,11 @@ def from_packing_results(
                     f"bin_bounds={bin_bounds!r} for data[0] "
                     f"but {pr.bin_bounds!r} for data[{i}]?")
 
-        es_from_end_results((pr.end_result for pr in data), end_stats.append)
+        end_stats.extend(es_from_end_results(pr.end_result for pr in data))
         if len(end_stats) != 1:
             raise ValueError(f"got {end_stats} from {data}?")
 
-        consumer(PackingStatistics(
+        yield PackingStatistics(
             end_statistics=end_stats[0],
             n_items=n_items,
             n_different_items=n_different_items,
@@ -295,7 +292,7 @@ def from_packing_results(
             },
             objective_bounds=objective_bounds,
             bin_bounds=bin_bounds,
-        ))
+        )
         end_stats.clear()
 
 
@@ -311,200 +308,175 @@ def to_csv(results: Iterable[PackingStatistics], file: str) -> Path:
     logger(f"Writing packing statistics to CSV file {path!r}.")
     path.ensure_parent_dir_exists()
     with path.open_for_write() as wt:
-        csv_write(
-            data=sorted(results), consumer=line_writer(wt),
-            setup=CsvWriter().setup,
-            get_column_titles=CsvWriter.get_column_titles,
-            get_row=CsvWriter.get_row,
-            get_header_comments=CsvWriter.get_header_comments,
-            get_footer_comments=CsvWriter.get_footer_comments,
-            get_footer_bottom_comments=CsvWriter.get_footer_bottom_comments)
+        consumer: Final[Callable[[str], None]] = line_writer(wt)
+        for p in CsvWriter.write(sorted(results)):
+            consumer(p)
     logger(f"Done writing packing statistics to CSV file {path!r}.")
     return path
 
 
-def from_csv(file: str,
-             consumer: Callable[[PackingStatistics], None]) -> None:
+def from_csv(file: str) -> Iterable[PackingStatistics]:
     """
     Load the packing statistics from a CSV file.
 
     :param file: the file to read from
-    :param consumer: the consumer for the statistics
+    :returns: the iterable with the packing statistics
     """
-    if not callable(consumer):
-        raise type_error(consumer, "consumer", call=True)
     path: Final[Path] = file_path(file)
     logger(f"Now reading CSV file {path!r}.")
     with path.open_for_read() as rd:
-        csv_read(rows=rd,
-                 setup=CsvReader,
-                 parse_row=CsvReader.parse_row,
-                 consumer=consumer)
+        yield from CsvReader.read(rd)
     logger(f"Done reading CSV file {path!r}.")
 
 
-class CsvWriter:
+class CsvWriter(CsvWriterBase[PackingStatistics]):
     """A class for CSV writing of :class:`PackingStatistics`."""
 
-    def __init__(self, scope: str | None = None) -> None:
+    def __init__(self, data: Iterable[PackingStatistics],
+                 scope: str | None = None) -> None:
         """
         Initialize the csv writer.
 
+        :param data: the data to write
         :param scope: the prefix to be pre-pended to all columns
         """
-        #: an optional scope
-        self.scope: Final[str | None] = (
-            str.strip(scope)) if scope is not None else None
-
-        #: has this writer been set up?
-        self.__setup: bool = False
-        #: the end statistics writer
-        self.__es: Final[EsCsvWriter] = EsCsvWriter(scope)
-        #: the bin bounds
-        self.__bin_bounds: list[str] | None = None
-        #: the objectives
-        self.__objectives: list[SsCsvWriter] | None = None
-        #: the objective names
-        self.__objective_names: tuple[str, ...] | None = None
-        #: the lower bound names
-        self.__objective_lb_names: tuple[str, ...] | None = None
-        #: the upper bound names
-        self.__objective_ub_names: tuple[str, ...] | None = None
-
-    def setup(self, data: Iterable[PackingStatistics]) -> "CsvWriter":
-        """
-        Set up this csv writer based on existing data.
-
-        :param data: the data to setup with
-        :returns: this writer
-        """
-        if self.__setup:
-            raise ValueError("CSV writer has already been set up.")
-        self.__setup = True
-
         data = reiterable(data)
-        self.__es.setup(pr.end_statistics for pr in data)
+        super().__init__(data, scope)
+        #: the end statistics writer
+        self.__es: Final[EsCsvWriter] = EsCsvWriter((
+            pr.end_statistics for pr in data), scope)
 
         bin_bounds_set: Final[set[str]] = set()
         objectives_set: Final[set[str]] = set()
         for pr in data:
             bin_bounds_set.update(pr.bin_bounds.keys())
             objectives_set.update(pr.objectives.keys())
-        if set.__len__(bin_bounds_set) > 0:
-            self.__bin_bounds = sorted(bin_bounds_set)
+
+        #: the bin bounds
+        self.__bin_bounds: list[str] | None = None \
+            if set.__len__(bin_bounds_set) <= 0 else sorted(bin_bounds_set)
+
+        #: the objectives
+        objectives: list[SsCsvWriter] | None = None
+        #: the objective names
+        objective_names: tuple[str, ...] | None = None
+        #: the lower bound names
+        objective_lb_names: tuple[str, ...] | None = None
+        #: the upper bound names
+        objective_ub_names: tuple[str, ...] | None = None
         if set.__len__(objectives_set) > 0:
             p: Final[str | None] = self.scope
-            self.__objective_names = tuple(sorted(objectives_set))
-            self.__objective_lb_names = tuple(csv_scope(
-                oxx, _OBJECTIVE_LOWER) for oxx in self.__objective_names)
-            self.__objective_ub_names = tuple(csv_scope(
-                oxx, _OBJECTIVE_UPPER) for oxx in self.__objective_names)
-            self.__objectives = [SsCsvWriter(
+            objective_names = tuple(sorted(objectives_set))
+            objective_lb_names = tuple(csv_scope(
+                oxx, _OBJECTIVE_LOWER) for oxx in objective_names)
+            objective_ub_names = tuple(csv_scope(
+                oxx, _OBJECTIVE_UPPER) for oxx in objective_names)
+            objectives = [SsCsvWriter(
+                data=(ddd.objectives[k] for ddd in data),
                 scope=csv_scope(p, k), n_not_needed=True, what_short=k,
-                what_long=f"objective function {k}").setup(
-                    ddd.objectives[k] for ddd in data
-            ) for k in self.__objective_names]
+                what_long=f"objective function {k}") for k in objective_names]
 
-        return self
+        #: the objectives
+        self.__objectives: Final[list[SsCsvWriter] | None] = objectives
+        #: the objective names
+        self.__objective_names: Final[tuple[str, ...] | None] \
+            = objective_names
+        #: the lower bound names
+        self.__objective_lb_names: Final[tuple[str, ...] | None] \
+            = objective_lb_names
+        #: the upper bound names
+        self.__objective_ub_names: Final[tuple[str, ...] | None] \
+            = objective_ub_names
 
-    def get_column_titles(self, dest: Callable[[str], None]) -> None:
-        """
-        Get the column titles.
-
-        :param dest: the destination string consumer
-        """
+    def get_column_titles(self) -> Iterable[str]:
+        """Get the column titles."""
         p: Final[str | None] = self.scope
-        self.__es.get_column_titles(dest)
+        yield from self.__es.get_column_titles()
 
-        dest(csv_scope(p, KEY_BIN_HEIGHT))
-        dest(csv_scope(p, KEY_BIN_WIDTH))
-        dest(csv_scope(p, KEY_N_ITEMS))
-        dest(csv_scope(p, KEY_N_DIFFERENT_ITEMS))
+        yield csv_scope(p, KEY_BIN_HEIGHT)
+        yield csv_scope(p, KEY_BIN_WIDTH)
+        yield csv_scope(p, KEY_N_ITEMS)
+        yield csv_scope(p, KEY_N_DIFFERENT_ITEMS)
         if self.__bin_bounds:
             for b in self.__bin_bounds:
-                dest(csv_scope(p, b))
+                yield csv_scope(p, b)
         if self.__objective_names and self.__objectives:
             for i, o in enumerate(self.__objectives):
-                dest(csv_scope(p, self.__objective_lb_names[i]))
-                o.get_column_titles(dest)
-                dest(csv_scope(p, self.__objective_ub_names[i]))
+                yield csv_scope(p, self.__objective_lb_names[i])
+                yield from o.get_column_titles()
+                yield csv_scope(p, self.__objective_ub_names[i])
 
-    def get_row(self, data: PackingStatistics,
-                dest: Callable[[str], None]) -> None:
+    def get_row(self, data: PackingStatistics) -> Iterable[str]:
         """
         Render a single packing result record to a CSV row.
 
         :param data: the end result record
-        :param dest: the string consumer
+        :returns: the iterable with the row text
         """
-        self.__es.get_row(data.end_statistics, dest)
-        dest(repr(data.bin_height))
-        dest(repr(data.bin_width))
-        dest(repr(data.n_items))
-        dest(repr(data.n_different_items))
+        yield from self.__es.get_row(data.end_statistics)
+        yield repr(data.bin_height)
+        yield repr(data.bin_width)
+        yield repr(data.n_items)
+        yield repr(data.n_different_items)
         if self.__bin_bounds:
             for bb in self.__bin_bounds:
-                dest(repr(data.bin_bounds[bb])
-                     if bb in data.bin_bounds else "")
+                yield (repr(data.bin_bounds[bb])
+                       if bb in data.bin_bounds else "")
         if self.__objective_names and self.__objectives:
             lb: Final[tuple[str, ...] | None] = self.__objective_lb_names
             ub: Final[tuple[str, ...] | None] = self.__objective_ub_names
             for i, ob in enumerate(self.__objective_names):
                 if lb is not None:
                     ox = lb[i]
-                    dest(num_to_str(data.objective_bounds[ox])
-                         if ox in data.objective_bounds else "")
-                SsCsvWriter.get_optional_row(
-                    self.__objectives[i], data.objectives.get(ob), dest)
+                    yield (num_to_str(data.objective_bounds[ox])
+                           if ox in data.objective_bounds else "")
+                yield from SsCsvWriter.get_optional_row(
+                    self.__objectives[i], data.objectives.get(ob))
                 if ub is not None:
                     ox = ub[i]
-                    dest(num_to_str(data.objective_bounds[ox])
-                         if ox in data.objective_bounds else "")
+                    yield (num_to_str(data.objective_bounds[ox])
+                           if ox in data.objective_bounds else "")
 
-    def get_header_comments(self, dest: Callable[[str], None]) -> None:
+    def get_header_comments(self) -> Iterable[str]:
         """
         Get any possible header comments.
 
-        :param dest: the destination
+        :returns: the header comments
         """
-        dest("End Statistics of Bin Packing Experiments")
-        dest("See the description at the bottom of the file.")
+        return ("End Statistics of Bin Packing Experiments",
+                "See the description at the bottom of the file.")
 
-    def get_footer_comments(self, dest: Callable[[str], None]) -> None:
+    def get_footer_comments(self) -> Iterable[str]:
         """
         Get any possible footer comments.
 
-        :param dest: the destination
+        :returns: the footer comments
         """
-        self.__es.get_footer_comments(dest)
-        dest("")
+        yield from self.__es.get_footer_comments()
+        yield ""
         p: Final[str | None] = self.scope
         if self.__bin_bounds:
             for bb in self.__bin_bounds:
-                dest(f"{csv_scope(p, bb)} is a lower "
-                     "bound for the number of bins.")
+                yield (f"{csv_scope(p, bb)} is a lower "
+                       "bound for the number of bins.")
         if self.__objectives and self.__objective_names:
             for i, obb in enumerate(self.__objective_names):
                 ob: str = csv_scope(p, obb)
                 ox: str = csv_scope(ob, _OBJECTIVE_LOWER)
-                dest(f"{ox}: a lower bound of the {ob} objective function.")
-                self.__objectives[i].get_footer_comments(dest)
+                yield f"{ox}: a lower bound of the {ob} objective function."
+                yield from self.__objectives[i].get_footer_comments()
                 ox = csv_scope(ob, _OBJECTIVE_UPPER)
-                dest(f"{ox}: an upper bound of the {ob} objective function.")
+                yield f"{ox}: an upper bound of the {ob} objective function."
 
-    def get_footer_bottom_comments(self, dest: Callable[[str], None]) -> None:
-        """
-        Get the bottom footer comments.
-
-        :param dest: the destination
-        """
-        motipyapps_footer_bottom_comments(
-            self, dest, "The packing data is assembled using module "
-                        "moptipyapps.binpacking2d.packing_statistics.")
-        EsCsvWriter.get_footer_bottom_comments(self.__es, dest)
+    def get_footer_bottom_comments(self) -> Iterable[str]:
+        """Get the bottom footer comments."""
+        yield from motipyapps_footer_bottom_comments(
+            self, "The packing data is assembled using module "
+                  "moptipyapps.binpacking2d.packing_statistics.")
+        yield from EsCsvWriter.get_footer_bottom_comments(self.__es)
 
 
-class CsvReader:
+class CsvReader(CsvReaderBase[PackingStatistics]):
     """A class for CSV parsing to get :class:`PackingStatistics`."""
 
     def __init__(self, columns: dict[str, int]) -> None:
@@ -513,7 +485,7 @@ class CsvReader:
 
         :param columns: the columns
         """
-        super().__init__()
+        super().__init__(columns)
         #: the end result csv reader
         self.__es: Final[EsCsvReader] = EsCsvReader(columns)
         #: the index of the n-items column
@@ -600,15 +572,11 @@ if __name__ == "__main__":
     args: Final[argparse.Namespace] = parser.parse_args()
 
     src_path: Final[Path] = args.source
-    packing_results: Final[list[PackingResult]] = []
+    packing_results: Iterable[PackingResult]
     if src_path.is_file():
         logger(f"{src_path!r} identifies as file, load as end-results csv")
-        pr_from_csv(src_path, packing_results.append)
+        packing_results = pr_from_csv(src_path)
     else:
         logger(f"{src_path!r} identifies as directory, load it as log files")
-        pr_from_logs(src_path, packing_results.append)
-
-    packing_stats: Final[list[PackingStatistics]] = []
-    from_packing_results(
-        results=packing_results, consumer=packing_stats.append)
-    to_csv(packing_stats, args.dest)
+        packing_results = pr_from_logs(src_path)
+    to_csv(from_packing_results(results=packing_results), args.dest)
