@@ -18,7 +18,7 @@ from typing import Any, Callable, Final, Iterable, cast
 import numpy as np
 from moptipy.algorithms.so.vector.cmaes_lib import BiPopCMAES
 from moptipy.api.execution import Execution
-from moptipy.api.experiment import Parallelism, run_experiment
+from moptipy.api.experiment import run_experiment
 from moptipy.api.process import Process
 from pycommons.io.path import Path, directory_path
 
@@ -53,10 +53,9 @@ def make_instances() -> Iterable[Callable[[], Instance]]:
         controllers.extend(partially_linear(system))
         controllers.extend(predefined(system))
         controllers.extend(peaks(system))
-        for controller in controllers:
-            res.append(cast(
-                Callable[[], Instance],
-                lambda _s=system, _c=controller: Instance(_s, _c)))
+        res.extend(cast(
+            "Callable[[], Instance]", lambda _s=system, _c=controller:
+            Instance(_s, _c)) for controller in controllers)
     return res
 
 
@@ -91,11 +90,11 @@ def on_completion(instance: Any, log_file: Path, process: Process) -> None:
     :param log_file: the log file
     :param process: the process
     """
-    inst: Final[Instance] = cast(Instance, instance)
+    inst: Final[Instance] = cast("Instance", instance)
     dest_dir: Final[Path] = directory_path(dirname(log_file))
     base_name: str = basename(log_file)
     base_name = base_name[:base_name.rindex(".")]
-    result: np.ndarray = cast(np.ndarray, process.create())
+    result: np.ndarray = cast("np.ndarray", process.create())
     process.get_copy_of_best_x(result)
     j: Final[float] = process.get_best_f()
     inst.describe_parameterization(f"F = {j}", result, base_name, dest_dir)
@@ -121,7 +120,6 @@ def run(base_dir: str, n_runs: int = 5) -> None:
         instances=instances,
         setups=[cmaes],
         n_runs=n_runs,
-        n_threads=Parallelism.PERFORMANCE,
         perform_warmup=False,
         perform_pre_warmup=False,
         on_completion=on_completion)
