@@ -107,8 +107,8 @@ class Instance(QAPInstance):
                 f = flows[i, j]
                 if f > horizon:
                     continue
-                flow_matrix[i, j] = int(round(
-                    multiplier * ((max_val - f + 1) ** flow_power)))
+                flow_matrix[i, j] = round(
+                    multiplier * ((max_val - f + 1) ** flow_power))
 
         # construct a name if necessary
         if name is None:
@@ -300,7 +300,7 @@ class Instance(QAPInstance):
             raise type_error(tag_titles, "tag_titles", Iterable)
 
         # build a distance matrix and purge all zero-distance elements
-        datal: list[T] = cast(list[T], data) \
+        datal: list[T] = cast("list[T]", data) \
             if isinstance(data, list) else list(data)
         mappings: list[tuple[T, int]] = []
         distances: list[list[int | float]] = []
