@@ -63,7 +63,7 @@ class OrderingSpace(Permutations):
         text: list[str] = []
 
         text.extend(super().to_str(x).split("\n"))
-        text.append("")
+        text.append("")  # noqa
 
         row: list[str] = [_ZERO_BASED_INDEX, _SUGGESTED_X_IN_0_1]
         row.extend(self.instance.tag_titles)
@@ -71,8 +71,7 @@ class OrderingSpace(Permutations):
 
         for tag, i in tags:
             row.clear()
-            row.append(str(x[i]))
-            row.append(float_to_str(x[i] / n))
+            row.extend((str(x[i]), float_to_str(x[i] / n)))
             row.extend(tag)
             text.append(f"{CSV_SEPARATOR.join(row)}")
         return "\n".join(text)
