@@ -465,7 +465,7 @@ class Instance(Component):
         container: Final = Instance.from_resource
         inst_attr: Final[str] = f"__inst_{name}"
         if hasattr(container, inst_attr):  # instance loaded?
-            return cast(Instance, getattr(container, inst_attr))
+            return cast("Instance", getattr(container, inst_attr))
 
         lb: int | None = _BOUNDS.get(name, None)
         with open_resource_stream(f"{name}.dat") as stream:
